function X_out = tmprod(X,U,mode)
        %----------------------------------------------------------------------------------
        % This function computes the mode-n product between a 3rd order tensor and a matrix
        %----------------------------------------------------------------------------------
        % INPUTS: - tensor X of size (IxJxK)
        %         - Matrix U with I columns (if mode==1) or J columns (if mode==2) or K columns (if mode==3)
        %         - mode= 1, 2 or 3
        %----------------------------------------------------------------------------------
        % OUTPUTS: X_out results from mode-n (n=1,2 or 3) product between X and U
        %----------------------------------------------------------------------------------
        [I,J,K]=size(X);
        [M,N]=size(U);
        
        if (mode~=1) & (mode~=2) & (mode~=3)
            error('The input variable mode should be 1, 2 or 3')
        end
        
        if N~=size(X,mode) 
            error(['The number of columns of the input matrix should be equal to dimension ',int2str(mode),' of the input tensor'])
        end    

        if mode==1
            X_out = reshape(U*reshape(X,I,J*K) ,M,J,K);      
        elseif mode==2
            X_out = permute(reshape (U*reshape(permute(X,[2 1 3]),J,I*K), M,I,K),[2 1 3]);        
        elseif mode==3
            X_out = permute(reshape (U*reshape(permute(X,[3 1 2]),K,I*J), M,I,J),[2 3 1]);
        end
